﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using EPiServer.Core;
using System.Text;
using EPiServer.Util;
using EPiServer.UI;
using EPiServer.Web.PageExtensions;
using EPiServer.Core.Html;
using EPiServer.XForms.Util;
using EPiServer.XForms;
using log4net;
using System.Collections;
using System.Xml;
using EPiServer.Web.PropertyControls;
using EPiServer.ClientScript.Events;
using EPiServer.ClientScript;
using EPiServer.Security;
using EPiServer.Configuration;
using System.Web.UI.HtmlControls;
using System.IO;
using EPiServer.UI.WebControls;
using EPiServer.UI.Edit;
using EPiServer.XForms.WebControls;
using EPiServer.Web.WebControls;
using UppsalaKommun.CommonLibrary.Web.Managers;

namespace UppsalaKommun.Insidan.epi.edit
{
    [CLSCompliant(false)]
    public partial class XFormEdit : SystemPageBase
    {
        // Fields
        private const string _cellContent = "&nbsp;";
        private const string _cellEnd = "<br />";
        private const string _idPrefix = "id_field";
        private XForm _xForm;
        protected ToolButton CancelButton;
        protected Literal CssPath;
        protected ToolButton DeleteButton;
        protected PlaceHolder ErrorMsg;
        protected ToolButton Export;
        protected XFormFieldProperty FieldProperty;
        protected FileUpload FileUploadControl;
        protected CheckBox FormAnonymousPost;
        protected TextBox FormContent;
        protected XFormControl FormControl;
        protected Tab FormFields;
        protected Panel FormFieldsPanel;
        protected ListBox FormFolders;
        protected CheckBox FormMultiplePost;
        protected TextBox FormName;
        protected InputPageReference FormPageAfterPost;
        protected HelpButton HelpButton;
        protected ToolButton Import;
        protected Tab ImportExport;
        protected Panel ImportExportPanel;
        protected System.Web.UI.WebControls.Label Label1;
        protected System.Web.UI.WebControls.Label Label2;
        protected System.Web.UI.WebControls.Label Label3;
        protected System.Web.UI.WebControls.Label Label4;
        protected System.Web.UI.WebControls.Label Label5;
        protected TabStrip menuTab;
        protected Literal NewFormContent;
        protected HtmlTableCell PlaceHolder;
        protected Panel PropertiesArea;
        protected ToolButton SaveAndClose;
        protected HtmlInputHidden SaveAsAction;
        protected ToolButton SaveAsButton;
        protected ToolButton SaveButton;
        protected Tab TableLayout;
        protected Panel TableLayoutPanel;
        protected Panel tabView;
        protected XFormToolbox Toolbox;
        protected Panel XformControl;

        // Methods
        public XFormEdit()
            : base(0, SiteRedirect.OptionFlag)
        {
        }

        protected void CancelButton_Click(object sender, EventArgs e)
        {
            this.RedirectScript("null");
        }

        private bool ConvertForm()
        {
            string html = null;
            string str3 = null;
            try
            {
                HtmlParser parser = new HtmlParser(this.FormContent.Text.Replace("CHECKED", "checked=\"true\"").Replace("selected", "selected=\"true\""), false);
                html = parser.ToString().Replace("&nbsp;", "&#160;");
                this.XForm.Document = XFormConvert.Transform(html);
                string format = "<%@ Register TagPrefix='xforms' Namespace='EPiServer.XForms.WebControls' Assembly='EPiServer.XForms' %><xforms:XFormControl Runat='Server'>{0}</xforms:XFormControl>";
                this.Page.ParseControl(string.Format(format, this.XForm.Document.InnerXml)).RenderControl(new HtmlTextWriter(new StringWriter()));
            }
            catch (XFormException exception)
            {
                this.ShowMessage(string.Format(base.Translate(exception.Message), (string)exception.Param), true);
                this.NewFormContent.Text = html;
                return false;
            }
            catch (Exception exception2)
            {
                LogManager.GetLogger(base.GetType()).Error(string.Format("Invalid Xformular:\n {0}\nParsed XForm:\n{1}", html, str3), exception2);
                this.ShowMessage(base.Translate("/edit/editxform/invalid"), true);
                this.NewFormContent.Text = html;
                return false;
            }
            return true;
        }

        protected void DeleteButton_Click(object sender, EventArgs e)
        {
            if (this._xForm != null)
            {
                this._xForm.Delete();
                XFormManager xformDeleter = new XFormManager(); //Uppdaterar cachen.
                xformDeleter.DeleteForm(this.XForm);
                
            }
            this.RedirectScript("null");
        }

        protected void ExportForm_Click(object sender, EventArgs e)
        {
            if (!this.ConvertForm())
            {
                this.SetVisibilityOnFailedForm();
            }
            else
            {
                string str2;
                string s = string.IsNullOrEmpty(this.XForm.FormName) ? "xform" : this.XForm.FormName;
                base.Response.Clear();
                base.Response.ContentType = "application/octet-stream";
                base.Response.Charset = "utf-8";
                if (base.Request.Browser.Browser == "Firefox")
                {
                    str2 = s;
                }
                else
                {
                    str2 = base.Server.UrlEncode(s).ToString().Replace("+", "%20");
                }
                base.Response.AddHeader("Content-Disposition", "attachment; filename=\"" + str2 + ".xml\"");
                base.Response.Write(this.XForm.Document.InnerXml);
                base.Response.End();
            }
        }

        private ListBox GetSortedList(ListBox list)
        {
            SortedList list2 = new SortedList();
            foreach (ListItem item in list.Items)
            {
                list2.Add(item.Value.ToString(), item.Text.ToString());
            }
            list.Items.Clear();
            foreach (DictionaryEntry entry in list2)
            {
                ListItem item2 = new ListItem();
                item2.Text = entry.Value.ToString();
                item2.Value = entry.Key.ToString();
                list.Items.Add(item2);
            }
            return list;
        }

        protected void ImportForm_Click(object sender, EventArgs e)
        {
            if (this.FileUploadControl.HasFile)
            {
                try
                {
                    XmlTextReader reader = new XmlTextReader(this.FileUploadControl.PostedFile.InputStream);
                    this.XForm.Document.Load(reader);
                    this.FormControl.Data = this.XForm.CreateFormData();
                }
                catch (XmlException exception)
                {
                    LogManager.GetLogger(base.GetType()).Error("Could not import XForm:\n", exception);
                    this.ShowMessage(base.Translate("/edit/editxform/errorimporting"), true);
                    return;
                }
                this.FormControl.DataBind();
            }
        }

        private void InitializeFields()
        {
            foreach (XFormFolder folder in XFormFolder.GetFolders())
            {
                this.FormFolders.Items.Add(new ListItem(folder.Name, folder.SubPath));
            }
            this.FormFolders = this.GetSortedList(this.FormFolders);
            ListItem item = new ListItem();
            item.Text = base.TranslateForScript("/edit/editxform/nofolder");
            item.Value = "/";
            this.FormFolders.Items.Add(item);
            if (this.FormFolders.Items.Count > 0)
            {
                ListItem item2 = this.FormFolders.Items.FindByValue(Convert.ToString(base.Request.QueryString["selectedFormName"]));
                if (item2 != null)
                {
                    item2.Selected = true;
                }
                else
                {
                    item.Selected = true;
                }
            }
            else
            {
                item.Selected = true;
            }
            if (this.XForm != null)
            {
                this.FormName.Text = this.XForm.FormName;
                this.FormFolders.SelectedValue = this.XForm.Folder.SubPath;
                this.FormAnonymousPost.Checked = this.XForm.AllowAnonymousPost;
                this.FormMultiplePost.Checked = this.XForm.AllowMultiplePost;
                this.DeleteButton.Visible = this.XForm != null;
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            base.Title = LanguageManager.Instance.Translate("/edit/editxform/heading");
            base.RegisterClientScriptFile(base.ResolveUrlFromUI("javascript/xformedit.js"));
            if ((base.Request["formid"] != null) && (base.Request["formid"].Length > 0))
            {
                this.XForm = XForm.CreateInstance(new Guid(base.Request["formid"].Replace("'", string.Empty)));
                this.WritePagesWithThisForm(this.XForm);
            }
            else
            {
                this.DeleteButton.Enabled = false;
                this.SaveAsButton.Enabled = false;
                if (base.IsPostBack)
                {
                    this.NewFormContent.Text = string.Empty;
                }
                else
                {
                    this.NewFormContent.Text = "<table id=\"id_matrix\"><tr><td valign=\"top\">&nbsp;</td></tr></table>";
                }
            }
            this.FormControl.FormDefinition = this.XForm;
            this.FormControl.DataBind();
            this.WriteEditFormScript();
            if (!base.IsPostBack)
            {
                this.SetPageAfterPostForEdit();
            }
            string cssPathsForPage = PropertyLongStringControl.GetCssPathsForPage(this.CurrentPage);
            if (cssPathsForPage.Length > 0)
            {
                foreach (string str2 in cssPathsForPage.Split(new char[] { ',' }))
                {
                    base.RegisterCSSFile(str2, str2);
                }
            }
            ThemeUtility.RegisterThemeCss(this.Page, "xformedit.css");
            this.RegisterScript();
            if (!base.IsPostBack)
            {
                this.InitializeFields();
            }
            if (this.SaveAsAction.Value == "1")
            {
                this.XForm = XForm.CreateInstance();
                this.SaveForm(true);
            }
        }

        private void RedirectScript(string returnValue)
        {
            base.ScriptManager.AddEventListener(this.Page, new CustomEvent(EventType.Load, string.Format("function(e) {{EPi.GetDialog().Close({0});}}", returnValue)));
        }

        private void RefreshXForm(string xFormID)
        {
            string url = base.ResolveUrl("XFormEdit.aspx");
            url = url + "?pageId=" + base.Request["pageId"] + "&parentId=" + base.Request["parentId"] + "&formid=" + xFormID + "&selectedFormName=" + this.FormFolders.SelectedValue.ToString();
            base.Response.Redirect(url);
        }

        private void RegisterScript()
        {
            this.Page.ClientScript.RegisterStartupScript(base.GetType(), "formvalidation", string.Format("\r\n\t\t\t<script type='text/javascript'>\r\n            //<![CDATA[\r\n            var xFormControl = document.getElementById('FormControl');\r\n            var formContent = document.getElementById('{0}');\r\n\t\t\t\r\n\t\t\tfunction formValid()\r\n\t\t\t{{\r\n\t\t\t\treturn true;\r\n\t\t\t}}\r\n\t\t\t//]]>\r\n\t\t\t</script>", this.FormContent.ClientID));
        }

        protected void SaveAndCloseButton_Click(object sender, EventArgs e)
        {
            this.SaveForm(true);
        }

        protected void SaveButton_Click(object sender, EventArgs e)
        {
            this.SaveForm(false);
        }

        private void SaveForm(bool iClose)
        {
            this.SetMetaData(this.XForm);
            if (this.XForm.FormName.Trim().Length == 0)
            {
                this.ShowMessage(base.Translate("/edit/editxform/formnameempty"), true);
                this.SetVisibilityOnFailedForm();
            }
            else if (!this.ConvertForm())
            {
                this.SetVisibilityOnFailedForm();
            }
            else
            {
                this.XForm.Save();
                XFormManager xformSaver = new XFormManager(); //Uppdaterar cachen.
                
                XFormFolder.Move(this.XForm.Id, XFormFolder.Create(this.FormFolders.SelectedValue));
                this.XForm = XForm.CreateInstance((Guid)this.XForm.Id); //Trying to get current folder to work /Daniel
                xformSaver.SaveForm(this.XForm);
                if (iClose)
                {
                    this.RedirectScript(string.Format("'{0}'", this.XForm.Id.ToString()));
                }
                else
                {
                    this.RefreshXForm(this.XForm.Id.ToString());
                }
            }
        }

        private void SetMetaData(XForm xForm)
        {
            xForm.FormName = this.FormName.Text;
            if (xForm.CreatedBy == null)
            {
                xForm.CreatedBy = PrincipalInfo.Current.Name;
                xForm.Created = DateTime.Now;
            }
            xForm.Changed = DateTime.Now;
            xForm.ChangedBy = PrincipalInfo.Current.Name;
            xForm.AllowAnonymousPost = this.FormAnonymousPost.Checked;
            xForm.AllowMultiplePost = this.FormMultiplePost.Checked;
            xForm.PageAfterPost = PageReference.IsNullOrEmpty(this.FormPageAfterPost.PageLink) ? 0 : this.FormPageAfterPost.PageLink.ID;
            xForm.MailSubject = string.Format(base.Translate("/edit/editxform/formdatamailsubject"), xForm.FormName, Settings.Instance.SiteDisplayName);
        }

        private void SetPageAfterPostForEdit()
        {
            if (this.XForm != null)
            {
                if (this.XForm.PageAfterPost > 0)
                {
                    this.FormPageAfterPost.PageLink = new PageReference(this.XForm.PageAfterPost);
                }
                else
                {
                    this.FormPageAfterPost.PageLink = PageReference.EmptyReference;
                }
            }
        }

        private void SetVisibilityOnFailedForm()
        {
            this.NewFormContent.Visible = true;
            this.FormControl.Visible = false;
        }

        private void ShowMessage(string msg, bool isWarning)
        {
            System.Web.UI.WebControls.Label child = new System.Web.UI.WebControls.Label();
            HtmlTable table = new HtmlTable();
            HtmlTableRow row = new HtmlTableRow();
            HtmlTableCell cell = new HtmlTableCell();
            HtmlTableCell cell2 = new HtmlTableCell();
            child.CssClass = "EP-systemMessage";
            cell2.InnerText = msg;
            if (isWarning)
            {
                HtmlImage image = new HtmlImage();
                image.Src = ThemeUtility.GetImageThemeUrl(this, "Tools/Warning.gif");
                cell.Controls.Add(image);
                row.Controls.Add(cell);
            }
            row.Controls.Add(cell2);
            table.Controls.Add(row);
            child.Controls.Add(table);
            this.ErrorMsg.Controls.Add(child);
            this.ErrorMsg.Visible = true;
        }

        private void WriteEditFormScript()
        {
            this.Page.ClientScript.RegisterStartupScript(base.GetType(), "formscript", string.Format("\r\n                <script type='text/javascript'>\r\n                //<![CDATA[\r\n                sCellContent\t\t= '{0}';\r\n                sCellEnd\t\t\t= '{1}';\r\n                sIdPrefix\t\t\t= '{2}';\r\n                nId\t\t\t\t\t= {3};\r\n\r\n                sRadioProperties\t= '{4}';\r\n                sCheckboxProperties = '{5}';\r\n                sSelectProperties\t= '{6}';\r\n                sFieldExistsAlert\t= '{7}';\r\n                deleteRowConfirmMessage\t= '{8}';\r\n                deleteCellConfirmMessage = '{9}';\r\n                //]]>\r\n                </script>", new object[] { "&nbsp;", "<br />", "id_field", 2, base.TranslateForScript("/edit/editform/radiobuttonproperties"), base.TranslateForScript("/edit/editform/checkboxproperties"), base.TranslateForScript("/edit/editform/selectboxproperties"), base.TranslateForScript("/edit/editform/fieldexists"), base.TranslateForScript("/edit/editform/deleterowconfirmmessage"), base.TranslateForScript("/edit/editform/deletecolumnconfirmmessage") }));
        }

        private void WritePagesWithThisForm(XForm form)
        {
            int num;
            PageDataCollection datas = XFormHelper.ListFormReferences(form);
            StringBuilder builder = new StringBuilder();
            try
            {
                num = Convert.ToInt32(base.Request["pageId"]);
            }
            catch (Exception)
            {
                num = -1;
            }
            bool flag = false;
            int num2 = 0;
            foreach (PageData data in datas)
            {
                if (data.PageLink.ID != num)
                {
                    if (num2++ != 0)
                    {
                        builder.Append(", ");
                    }
                    builder.Append(data.PageName).Append(" [").Append(data.PageLink.ID).Append("]");
                    flag = true;
                }
            }
            if (flag)
            {
                this.ShowMessage(base.Translate("/edit/editxform/existonpages") + " " + builder.ToString(), false);
                this.DeleteButton.Enabled = false;
                this.SaveButton.Attributes.Add("onclick", string.Format("if(formValid() &  confirm('{0}\\n{1}'))\r\n{{ \r\n formPopulateForSubmit(this.form);\r\nreturn true;\r\n }}\r\n else \r\n {{ \r\n return false; \r\n }}", base.TranslateForScript("/edit/editxform/savewarninginfo"), base.TranslateForScript("/edit/editxform/savewarningquestion")));
            }
        }

        // Properties
        protected XForm XForm
        {
            get
            {
                if (this._xForm == null)
                {
                    this._xForm = XForm.CreateInstance();
                }
                return this._xForm;
            }
            set
            {
                this._xForm = value;
            }
        }
    }


}
