﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using log4net;
using EPiServer.XForms;
using System.Diagnostics;
using System.Web.Caching;
using System.Text;
using EPiServer.Core;
using EPiServer.Util;

namespace UppsalaKommun.CommonLibrary.Web.Managers
{
    public class XFormManager
    {
        private const string _cachekey = "XformsSelectCachekey12309k";
        private const string cacheTimeConstantsKey = "XFormsCacheTimeInMinutes";
        private int _cacheTimeInMinutes = 1200;
        
        public IList<XForm> GetForms()
        {
            return GetForms(null);
        }
        public XFormManager()
        {
            
        }
        public string GetxFormUsedonPages(string formId)
        {
            try
            {
                string cacheKey = "xFormUsedonPages" + formId;
                object cacheItem = HttpContext.Current.Cache[cacheKey];
               
                if (cacheItem == null)
                {
                   
                    StringBuilder str = new StringBuilder();
                    PageDataCollection datas = XFormHelper.ListFormReferences(formId);
                    for (int i = 0; i < datas.Count; i++)
                    {
                        str.Append(string.Format("{0}[{1}]<br />", datas[i].PageName.ToString(), datas[i].PageLink.ToString()));
                    }
                    if (str.Length > 0)
                    {
                        str.Insert(0, "<strong>");
                        str.Append("</strong>");
                    }
                    string result = str.ToString();
                    HttpContext.Current.Cache.Insert(cacheKey, result, null, DateTime.Now.AddMinutes(_cacheTimeInMinutes), Cache.NoSlidingExpiration);
                    return result;
                }
                else
                {
                    string result = (string)cacheItem;
                    return result;
                }
            }
            catch (Exception ex)
            {
                //Do some logging with log4net...
            }
            return string.Empty;
        }
        public IList<XForm> GetForms(string folder)
        {
            try
            {
                object forms = HttpContext.Current.Cache[_cachekey];
                IList<XForm> formList = null;
                if (forms == null)
                {
                    formList = XFormFolder.GetForms();
                    HttpContext.Current.Cache.Insert(_cachekey, formList, null, DateTime.Now.AddMinutes(_cacheTimeInMinutes), System.Web.Caching.Cache.NoSlidingExpiration);
                   
                }
                else
                {
                    formList = forms as IList<XForm>;
                }
                if (formList != null)
                {
                    if (string.IsNullOrEmpty(folder))
                    {
                        return formList;
                    }

                    var selectedForms = from form in formList
                                        where string.Compare("/" + form.Folder.Name, folder, true) == 0
                                        select form;
                    return selectedForms.ToList<XForm>();
                }
            }
            catch (Exception ex)
            {
                //Do some logging...
            }
            return new List<XForm>();
        }
        public void SaveForm(XForm form)
        {
            try
            {
                IList<XForm> forms = GetAllFormsExceptThis(form);
                forms.Add(form);
                HttpContext.Current.Cache.Insert(_cachekey, forms, null, DateTime.Now.AddMinutes(_cacheTimeInMinutes), System.Web.Caching.Cache.NoSlidingExpiration);
            }
            catch (Exception ex)
            {
                //Do some logging...
            }
        }
        public void DeleteForm(XForm form)
        {
            try
            {
                IList<XForm> forms = GetAllFormsExceptThis(form);
                HttpContext.Current.Cache.Insert(_cachekey, forms, null, DateTime.Now.AddMinutes(_cacheTimeInMinutes), System.Web.Caching.Cache.NoSlidingExpiration);
            }
            catch (Exception ex)
            {
                //Do some logging...
            }
        }
        private IList<XForm> GetAllFormsExceptThis(XForm form)
        {
            try
            {
                IList<XForm> forms = GetForms();
                IEnumerable<XForm> selectedForms = from formItem in forms
                                                   where (Guid)formItem.Id != (Guid)form.Id
                                                   select formItem;
                forms = selectedForms.ToList<XForm>();
                return forms;
            }
            catch (Exception ex)
            {
                //Do some logging...
            }
            return new List<XForm>();
        }
    }
}
